#!/usr/bin/env python
# -*- coding: UTF-8 -*

###############################################################################
# Copyright (C), 2015, TP-LINK Technologies Co., Ltd.
#
# Filename   : archerc2_upgrade.py
# Version    : 1.0.1
# Description: Module for TP-LINK system tools control.
# Author     : huangxin
# History:
#   1. 2015-12-09, huangxin, First created. 
###############################################################################
import os
import re
import time
import json
import subprocess
import getpass
from telnetlib import Telnet
import sys, socket, struct, select
from tmpv2 import TDPApplication, TMPApplication

import argparse, paramiko, bcrypt, _cffi_backend
import tmpv2, tmp_modules

import tkMessageBox
from Tkinter import *

import hashlib


ICMP_ECHO_REQUEST = 8 # Seems to be the same on Solaris.
SEQUENCE_NUMBER = 0
USERNAME = None
PASSWORD = None

def checksum(source_string):
    """
    I'm not too confident that this is right but testing seems
    to suggest that it gives the same answers as in_cksum in ping.c
    """
    sum = 0
    countTo = (len(source_string)/2)*2
    count = 0
    while count<countTo:
        thisVal = ord(source_string[count + 1])*256 + ord(source_string[count])
        sum = sum + thisVal
        sum = sum & 0xffffffff # Necessary?
        count = count + 2
 
    if countTo<len(source_string):
        sum = sum + ord(source_string[len(source_string) - 1])
        sum = sum & 0xffffffff # Necessary?
 
    sum = (sum >> 16)  +  (sum & 0xffff)
    sum = sum + (sum >> 16)
    answer = ~sum
    answer = answer & 0xffff
 
    # Swap bytes. Bugger me if I know why.
    answer = answer >> 8 | (answer << 8 & 0xff00)
 
    return answer
 
 
def receive_one_ping(my_socket, ID, timeout):
    """
    receive the ping from the socket.
    """
    timeLeft = timeout
    while True:
        startedSelect = time.time()
        whatReady = select.select([my_socket], [], [], timeLeft)
        howLongInSelect = (time.time() - startedSelect)
        if whatReady[0] == []: # Timeout
            return
 
        timeReceived = time.time()
        recPacket, addr = my_socket.recvfrom(1024)
        icmpHeader = recPacket[20:28]
        type, code, checksum, packetID, sequence = struct.unpack(
            "bbHHh", icmpHeader
        )
        if packetID == ID:
            bytesInDouble = struct.calcsize("d")
            timeSent = struct.unpack("d", recPacket[28:28 + bytesInDouble])[0]
            return timeReceived - timeSent
 
        timeLeft = timeLeft - howLongInSelect
        if timeLeft <= 0:
            return
 
 
def send_one_ping(my_socket, dest_addr, ID):
    """
    Send one ping to the given >dest_addr<.
    """
    global SEQUENCE_NUMBER
    dest_addr  =  socket.gethostbyname(dest_addr)
 
    # Header is type (8), code (8), checksum (16), id (16), sequence (16)
    my_checksum = 0
    
    # sequence number
    SEQUENCE_NUMBER = (SEQUENCE_NUMBER + 1) % 32768
 
    # Make a dummy heder with a 0 checksum.
    header = struct.pack("bbHHh", ICMP_ECHO_REQUEST, 0, my_checksum, ID, SEQUENCE_NUMBER)
    bytesInDouble = struct.calcsize("d")
    data = (192 - bytesInDouble) * "Q"
    data = struct.pack("d", time.time()) + data
 
    # Calculate the checksum on the data and the dummy header.
    my_checksum = checksum(header + data)
 
    # Now that we have the right checksum, we put that in. It's just easier
    # to make up a new header than to stuff it into the dummy.
    header = struct.pack(
        "bbHHh", ICMP_ECHO_REQUEST, 0, socket.htons(my_checksum), ID, SEQUENCE_NUMBER
    )
    packet = header + data
    my_socket.sendto(packet, (dest_addr, 1)) # Don't know about the 1
 
 
def do_one(dest_addr, timeout):
    """
    Returns either the delay (in seconds) or none on timeout.
    """
    icmp = socket.getprotobyname("icmp")
    try:
        my_socket = socket.socket(socket.AF_INET, socket.SOCK_RAW, icmp)
    except socket.error, (errno, msg):
        if errno == 1:
            # Operation not permitted
            msg = msg + (
                " - Note that ICMP messages can only be sent from processes"
                " running as root."
            )
            raise socket.error(errno, msg)
        raise socket.error(errno, msg) # raise the original error
 
    my_ID = os.getpid() & 0xFFFF
 
    send_one_ping(my_socket, dest_addr, my_ID)
    delay = receive_one_ping(my_socket, my_ID, timeout)
 
    my_socket.close()
    return delay
 
   
def check_ping(dest_addr, timeout = 2, count = 4):
    """
    Send >count< ping to >dest_addr< with the given >timeout< and return True or False.
    """
    response_num = ping(dest_addr, timeout, count)
    if response_num == count:
        return True
    else:
        return False
    
def ping(dest_addr, timeout = 2, count = 4):
    """
    Send >count< ping to >dest_addr< with the given >timeout< and return response package number.
    """
    response_num = 0
    for i in xrange(count):
        print "ping %s..." % dest_addr
        try:
            delay  =  do_one(dest_addr, timeout)
        except socket.gaierror, e:
            print "failed. (socket gaierror: '%s: %s')" % (e[0], e[1])
            continue
            # break
        except socket.error, e:
            print "failed. (socket error: '%s: %s')" % (e[0], e[1])
            continue
 
        if delay == None:
            print"failed. (timeout within %ssec.)" % timeout
        else:
            delay  =  delay * 1000
            print"get ping in %0.4fms" % delay
            response_num += 1
            
    loss = count - response_num
    loss_rate = 100*loss/count
    print "Packets: Sent = %s, Received = %s, Lost = %s (%s%% loss)" % (count, response_num, loss, loss_rate)
    
    return response_num
    

    
def wait_for_restart(ip):
    ''' Wait for finishing restarting.'''
    
    ## The full timeout is 24*1*5 = 120s
    ## ping ok, then wait 3s, ping ok, then wait 3s, ping ok
    RETRY_TIMES = 20
    retry_times = RETRY_TIMES
    time.sleep(10)
    print 'wait few seconds for restarting...'
    is_reboot = False
    # check if dut is reboot
    check_timeout = 15
    start_time    = time.time()
    while retry_times:
        time.sleep(3)
        if check_ping(ip, timeout=1, count=5):
            time.sleep(3)
            if check_ping(ip, timeout=1, count=5):
                time.sleep(3)
                if check_ping(ip, timeout=1, count=5):
                    time.sleep(3)
                    if check_ping(ip, timeout=1, count=5):
                        return True
                    else:
                        retry_times -= 1
                else:
                    retry_times -= 1
            else:
                retry_times -= 1
        else:
            retry_times -= 1
    
    # if not reboot break, else check reboot
    time.sleep(30)

def call_tmpcli(ip, opcode, input_file):
#    clipath = os.path.join(os.path.abspath('.'), "tmpcli")
#
#    process = subprocess.Popen([clipath,
#                                '-u', ip,
#                                '-U', USERNAME,
#                                '-P', PASSWORD,
#                                '-o', hex(opcode)] + args,
#                          stdout=subprocess.PIPE)
#    result, unused_err = process.communicate()
#    rc = process.poll()
#    if rc:
#        tkMessageBox.showerror("", "FAIL: Failed to execute tmpcli.")
#        raise RuntimeError("Failed to execute tmpcli")

    result = ""
    with TMPApplication(ip, None, timeout=30,
                        user=USERNAME, password=PASSWORD) as app:
        result = app.request(opcode, input=None,
                             infile=input_file, outfile=None)
                        
    result = json.loads(result)
    if result['error_code'] == 0:
        return result['result'] if 'result' in result else None
    else:
        raise RuntimeError("tmpcli returned error code {0}".format())

def upgrade_firmware(firmware_path, ip, wait_time=60, retry=16, interval=4):
    firmware_name = None
    if not os.path.exists(firmware_path):
        raise ValueError('the file %s not found' % firmware_path)
    else:
        firmware_name = os.path.basename(firmware_path)
    
    call_tmpcli(ip, 0xc407, firmware_path)
    
    print 'Please wait %s seconds for loading finished!'%wait_time
    time.sleep(wait_time)
    
    wait_for_restart(ip)
    
    print 'Upgrade finished.'


class upgradeFirmware(object):
    def __init__(self, master):
        self.master = master

        self.name = Label(master, text='username', borderwidth=2)
        self.name.grid(row=1, sticky=W)

        self.key  = Label(master, text='password', borderwidth=2)
        self.key.grid(row=2, sticky=W)

        self.svr  = Label(master, text='server', borderwidth=2)
        self.svr.grid(row=3, sticky=W)
        
        self.fir  = Label(master, text='firmware',borderwidth=2)
        self.fir.grid(row=4, sticky=W)

        self.nameEntry = Entry(master)
        self.nameEntry.grid(row=1, column=1, columnspan=2)
        self.nameEntry.focus_set()

        self.keyEntry = Entry(master)
        self.keyEntry.grid(row=2, column=1, columnspan=2)
        self.keyEntry.focus_set()

        self.svrEntry = Entry(master)
        self.svrEntry.grid(row=3, column=1, columnspan=2)
        self.svrEntry.focus_set()
        
        self.firEntry = Entry(master)
        self.firEntry.grid(row=4, column=1, columnspan=2)
        self.firEntry.focus_set()

        self.upgradeButton = Button(master, text='UPGRADE', borderwidth=2, command=self.upgrade)
        self.upgradeButton.grid(row=5, column=1)

    def upgrade(self):
        self.username = self.nameEntry.get()
        self.password = self.keyEntry.get()
        self.server   = self.svrEntry.get()
        self.firmware = self.firEntry.get()

        firmware_path = None
        firmware_type = self.firmware
        global USERNAME, PASSWORD

        USERNAME = hashlib.sha1(self.username).hexdigest()
#        PASSWORD = hashlib.sha1(self.password).hexdigest()
#        USERNAME = self.username
        PASSWORD = self.password

        userServer = self.server
        
        abs_path = os.path.abspath('.')
        dir_path_1st = os.path.dirname(abs_path)
        dir_path_2nd = os.path.dirname(dir_path_1st)
        dir_path_3th = os.path.dirname(dir_path_2nd)
        
        print dir_path_3th
        firmware_path = os.path.join(dir_path_3th, firmware_type)
        print firmware_path

        if USERNAME == None or PASSWORD == None or userServer == None or firmware_path == None:
            raise RuntimeError("invalid input!")

        upgrade_firmware(firmware_path=firmware_path, ip=userServer, wait_time=30)

        tkMessageBox.showinfo('UPGRADE','SUCCESS! ')

def main():
    root = Tk()
    root.title('UPGRADE FIRMWARE')
    myUpgrade = upgradeFirmware(root)
    mainloop()

if __name__ == '__main__':
    main()

'''
def main():
    server = None
    firmware_path = None
    firmware_type = "m5.bin"
    global USERNAME, PASSWORD
    
    file = open("profile")
    for line in file:
        line = line.strip()
        if line != "":
            args = line.split("=")
            if (args[0] == "username"):
                USERNAME = args[1]
            elif (args[0] == "password"):
                PASSWORD = args[1]
            elif (args[0] == "server"):
                server = args[1]
            elif (args[0] == "file"):
                firmware_path = os.path.join(os.path.abspath('.'), args[1])

    if USERNAME == None or PASSWORD == None or server == None or firmware_path == None:
        raise RuntimeError("profile invalid !")

    upgrade_firmware(firmware_path=firmware_path, ip=server, wait_time=30)


if __name__ == '__main__':
    main()
'''

